import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;

import formats.Formats;

import logger.Log;
import mscJNeuralNet.connectors.INetConnector;
import mscJNeuralNet.connectors.RandomSymmetryBreakingNetConnector;
import mscJNeuralNet.net.Net;
import mscJNeuralNet.net.PatternDoesNotMatchNetException;
import mscJNeuralNet.netPerformanceStatistics.NetPerformanceStatistics;
import mscJNeuralNet.patterns.Patterns;
import mscJNeuralNet.trainer.NetTrainer;
import mscJNeuralNet.trainingAlgorithms.INetTrainingAlgorithm;
import mscJNeuralNet.trainingAlgorithms.RProp;
import observerPattern.Observable;
import observerPattern.Observer;



public class TrainNN implements Observer{
	private String netpath;
	private int mInputsize;
	private int mHiddensize=0;
	private int mOutputsize=1;
	private ArrayList mAllMinNets = new ArrayList();
	private ArrayList mAllMinErrorsVal = new ArrayList();
	private ArrayList mAllMinCycles = new ArrayList();
	private ArrayList mAllAvgErrors = new ArrayList();
	private ArrayList mLocalMinNets;
	private ArrayList mLocalMinErrorsVal;
	private ArrayList mLocalMinCycles;
	private double mActualMinError;
	private Net mActualNet;
	private Net mActualMinNet;
	private int mActualMinCycle;
	private INetConnector mNetConnectionAlgo = new RandomSymmetryBreakingNetConnector();
	private INetTrainingAlgorithm mNetTrainAlgo = new RProp();
	private NetTrainer mTrainer = new NetTrainer();
	private int mIterations=10;
	private int mActualIteration=0;
	private String mTrainPat;
	private String mValPat;
	private int mMaxHiddenNeurons=16;
	
	public void initTrainer() throws PatternDoesNotMatchNetException {
		mTrainer.getObserverManager().removeObserver(this);
		int [] layerSizes;
		if (mHiddensize<1) {
			layerSizes = new int[2];
			layerSizes[0]=mInputsize;
			layerSizes[1]=mOutputsize;
		}
		else {
			layerSizes = new int[3];
			layerSizes[0]=mInputsize;
			layerSizes[1]=mHiddensize;
			layerSizes[2]=mOutputsize;
		}
		
		mActualNet = new Net(layerSizes);
		mNetConnectionAlgo.connectNet(mActualNet, null);
		mNetTrainAlgo.setNet(mActualNet);
		
		mTrainer.setTrainingAlgorithm(mNetTrainAlgo);
		mTrainer.setTargetErrorOnValidationPatterns(true);
		mTrainer.setTargetCycles(800);
		mTrainer.setTargetErrorType(NetPerformanceStatistics.ERRORTYPE_averageSumAbsoluteError);
		mTrainer.setTargetError(0.01);
		mTrainer.setUseRandomizedPatternOrder(false);
		mTrainer.setUpdateIntervall(1);
		mTrainer.setTrainingStateReport(false);
		mTrainer.setFinalReport(false);
	}
	
	public TrainNN(String PatTrain, String PatVal, int pIterations, String netpath) throws PatternDoesNotMatchNetException, IOException{
		this.netpath=netpath;
		mIterations=pIterations;
		// Dieses Programm als Observer des Trainers anmelden
		
		
		
		// 4. Trainieren des Netzes
		
		// TRAININGSDATEN
		// Trainingsdaten bereitstellen:
		// Trainingsdaten in ein Patterns Objekt bertragen
		// Komforbaler ist es, die Lerndaten in einer Textdatei 
		// zu speichern und hier zu laden
		mTrainPat=PatTrain;
		mValPat=PatVal;
		Patterns lPatTrain = Patterns.load(PatTrain);
		Patterns lPatVal = Patterns.load(PatVal);
		mInputsize = lPatTrain.getInputPatternSize();
//		mMaxHiddenNeurons = mInputsize-1;
//		System.out.println("contradictories:"+Formats.format(lPat.getContradictoryPatterns()));
		
		initTrainer();
		
		// Trainingsdatenmenge dem Trainer mitteilen
		mTrainer.setTrainingPatterns(lPatTrain);
		mTrainer.setValidationPatterns(lPatVal);
		
		mLocalMinNets = new ArrayList();
		mLocalMinErrorsVal = new ArrayList();
		mLocalMinCycles = new ArrayList();
		mTrainer.getObserverManager().addObserver(this);
		// Training beginnen
		mTrainer.start();
	}
	
	public static void main(String [] args) {
		try{
			new TrainNN("D:/diplom/_work/snnspatterns/strom-hybrid/stromhybrid-train.pat", "D:/diplom/_work/snnspatterns/strom-hybrid/stromhybrid-val.pat", 10, "D:/diplom/_work/snnspatterns/strom-hybrid/");
		}
		catch (Exception e){
			e.printStackTrace();
		}
	}

	/* (non-Javadoc)
	 * @see observerPattern.Observer#notify(observerPattern.Observable)
	 */
	public void notify(Observable pObservable) {
		// Diese Methode wird vom NetTrainer nach jedem Lernschritt aufgerufen.
		// Das bergebene Observable Objekt ist der NetTrainer selbst
		if (pObservable instanceof NetTrainer){
			NetTrainer lTrainer = (NetTrainer) pObservable;
			// Prfen, ob das Training vorbei ist
			if (!lTrainer.isRunning()){
//				System.out.println(mTrainer.getTrainingPatterns().getPatternsCount());
//				System.out.println(mTrainer.getValidationPatterns().getPatternsCount());
				Log.getInstance().writeln("iteration: "+mActualIteration+" minerr: "+mActualMinError+"@"+mActualMinCycle);
				mLocalMinNets.add(mActualMinNet);
				mLocalMinErrorsVal.add(new Double(mActualMinError));
				mLocalMinCycles.add(new Integer(mActualMinCycle));
				mActualIteration++;
				if (mActualIteration>=mIterations) {
					double lminerr = ((Double)mLocalMinErrorsVal.get(0)).doubleValue();
					int lminindex = 0;
					double lacterr;
					double lAvg = lminerr;
					for (int i=1; i<mLocalMinErrorsVal.size();i++) {
						lacterr = ((Double)mLocalMinErrorsVal.get(i)).doubleValue();
						lAvg+=lacterr;
						if (lminerr>lacterr) {
							lminindex=i;
							lminerr=lacterr;
						}
					}
					Log.getInstance().writeln("turn best: "+lminindex);
					Log.getInstance().writeln("err: "+lminerr+"@"+((Integer)mLocalMinCycles.get(lminindex)).intValue()+"");
					
					mAllMinErrorsVal.add(new Double(lminerr));
					mAllMinNets.add(mLocalMinNets.get(lminindex));
					mAllMinCycles.add(mLocalMinCycles.get(lminindex));
					try{
						Net.save(((Net)mLocalMinNets.get(lminindex)), new File(netpath+mInputsize+"x"+mHiddensize+"x"+mOutputsize+"-bestpost.net"));
					}
					catch (IOException e){
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					lAvg = (lAvg/mActualIteration);
					Log.getInstance().writeln("avg:"+lAvg);
					mAllAvgErrors.add(new Double(lAvg));
					mLocalMinNets = new ArrayList();
					mLocalMinErrorsVal = new ArrayList();
					mLocalMinCycles = new ArrayList();
					if (mHiddensize<1) mHiddensize=2;
					else if (mHiddensize<mMaxHiddenNeurons) mHiddensize++;
					else {
						// best of best
						double lAllMinE = 0;
						int lAllMinIndex = 0;
						double lActMinE;
						for (int i=0; i<mAllMinErrorsVal.size(); i++) {
							Log.getInstance().writeln("netstruct:"+mAllMinNets.get(i));
							Log.getInstance().writeln("mine:"+mAllMinErrorsVal.get(i)+"@"+mAllMinCycles.get(i));
							Log.getInstance().writeln("avg :"+mAllAvgErrors.get(i));
							if (i==0) {
								lAllMinE =((Double) mAllMinErrorsVal.get(i)).doubleValue();
								lAllMinIndex = i;
							}
							else {
								lActMinE = ((Double) mAllMinErrorsVal.get(i)).doubleValue();
								if (lActMinE<lAllMinE) {
									lAllMinE=lActMinE;
									lAllMinIndex = i;
								}
							}
						}
						Log.getInstance().writeln("Best:");
						Log.getInstance().writeln("netstruct:"+mAllMinNets.get(lAllMinIndex));
						Log.getInstance().writeln("mine:"+mAllMinErrorsVal.get(lAllMinIndex)+"@"+mAllMinCycles.get(lAllMinIndex));
						Log.getInstance().writeln("avg :"+mAllAvgErrors.get(lAllMinIndex));
						try {
							Net lBestNet = (Net) mAllMinNets.get(lAllMinIndex);
							Patterns lTrainPat = mTrainer.getTrainingPatterns();
							// evaluieren
							double [] lOutputs;
							double [] lAllOutputs = new double[lTrainPat.getPatternsCount()];
							for (int pat=0; pat<lTrainPat.getPatternsCount(); pat++) {
								lOutputs = lBestNet.getNetOutput(lTrainPat.getInputPattern(pat));
								lAllOutputs[pat] = lOutputs[0];
							}
							PrintWriter lOut = new PrintWriter(new BufferedWriter(new FileWriter(new File(mTrainPat+lBestNet+".txt"))));
							Log.getInstance().writeln(Formats.format(lAllOutputs));
							lOut.println("KNN output train");
							for (int pat=0; pat<lTrainPat.getPatternsCount(); pat++) {
								lOut.println(lAllOutputs[pat]);
							}
							lOut.flush();
							lOut.close();
							
							Patterns lPat = mTrainer.getValidationPatterns();
							lAllOutputs = new double[lPat.getPatternsCount()];
							for (int pat=0; pat<lPat.getPatternsCount(); pat++) {
								lOutputs = lBestNet.getNetOutput(lPat.getInputPattern(pat));
								lAllOutputs[pat] = lOutputs[0];
							}
							Log.getInstance().writeln(Formats.format(lAllOutputs));
							lOut = new PrintWriter(new BufferedWriter(new FileWriter(new File(mValPat+lBestNet+".txt"))));
							lOut.println("KNN output val");
							for (int pat=0; pat<lPat.getPatternsCount(); pat++) {
								lOut.println(lAllOutputs[pat]);
							}
							lOut.flush();
							lOut.close();
							
						}
						catch(Exception e) {
							e.printStackTrace();
						}
						System.exit(0);
					
					}
					mActualIteration=0;
					Log.getInstance().writeln("");
					Log.getInstance().writeln("----------------------");
					Log.getInstance().writeln("netstructure: "+mInputsize+"x"+mHiddensize+"x"+mOutputsize);
					Log.getInstance().writeln("----------------------");
				}
				Log.getInstance().writeln("");

				try{
					initTrainer();
					mTrainer.getObserverManager().addObserver(this);
					mTrainer.start(); 
				}
				catch (PatternDoesNotMatchNetException e){
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			else {
				if (lTrainer.getTrainingAlgorithm().getCycle()<2) { 
					mActualMinError = lTrainer.getLastCalculatedError();
					mActualMinNet = lTrainer.getTrainingAlgorithm().getNet().cloned();
					mActualMinCycle = lTrainer.getTrainingAlgorithm().getCycle();
				}
				else {
					if (mActualMinError>lTrainer.getLastCalculatedError()) {
						// Log.getInstance().writeln(lTrainer.getTrainingAlgorithm().getCycle()+" "+mActualMinError+">"+lTrainer.getLastCalculatedError());
						mActualMinError = lTrainer.getLastCalculatedError();
						mActualMinNet = lTrainer.getTrainingAlgorithm().getNet().cloned();
						mActualMinCycle = lTrainer.getTrainingAlgorithm().getCycle();
					}
				}
			}
		}
	}
}
