package mscJNeuralNet.connectors.enumerations;

import java.util.Enumeration;

import mscJNeuralNet.connectors.INetConnector;
import mscJNeuralNet.connectors.RandomNetConnector;
import mscJNeuralNet.connectors.RandomSymmetryBreakingNetConnector;
import mscJNeuralNet.connectors.WesselsBarnardNetConnector;

/**
 * Enumeration ber alle verfgbaren Klassen, welche die Schnittstelle
 * {@link mscJNeuralNet.connectors.INetConnector} implementieren.<br/>
 * Es werden die {@link mscJNeuralNet.connectors.INetConnector} in der folgenden Reihenfolge zurckgeliefert:
 * <dir>
 * 	<li>{@link RandomNetConnector}</li>
 *  <li>{@link RandomSymmetryBreakingNetConnector}</li>
 *  <li>{@link WesselsBarnardNetConnector}</li>
 * </dir>
 * <p/> 
 * Created on 09.06.2004
 * @author <A HREF="http://www.graviton.de">M. Serhat Cinar</A>
 * @version 09.06.2004
 * @see mscJNeuralNet.connectors.INetConnector
 * @see java.util.Enumeration
 */
public class NetConnectorsEnumeration implements Enumeration {
	
	private final INetConnector [] mConnectors = {
		new RandomNetConnector(),
		new RandomSymmetryBreakingNetConnector(),
		new WesselsBarnardNetConnector()
	};
	private int mIterationPosition = 0;
	
	/* (non-Javadoc)
	 * @see java.util.Enumeration#hasMoreElements()
	 */
	public boolean hasMoreElements() {
		return mConnectors.length > mIterationPosition;
	}

	/**
	 * Es wird jeweils ein Objekt des Typs {@link mscJNeuralNet.connectors.INetConnector} zurckgeliefert.
	 * 
	 * @see java.util.Enumeration#nextElement()
	 * @see INetConnector
	 */
	public Object nextElement() {
		return mConnectors[mIterationPosition++];
	}

}
