package mscJNeuralNet.examples;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JFrame;

import formats.Formats;

import observerPattern.Observable;
import observerPattern.Observer;

import mscJNeuralNet.connectors.INetConnector;
import mscJNeuralNet.connectors.RandomSymmetryBreakingNetConnector;
import mscJNeuralNet.gui.JNetErrorGraphView;
import mscJNeuralNet.gui.JNetInfoView;
import mscJNeuralNet.gui.JNetTrainerControl;
import mscJNeuralNet.gui.JNetTrainingAlgorithmControl;
import mscJNeuralNet.gui.JNetWeightView;
import mscJNeuralNet.net.Net;
import mscJNeuralNet.net.PatternDoesNotMatchNetException;
import mscJNeuralNet.netPerformanceStatistics.NetPerformanceReporter;
import mscJNeuralNet.netPerformanceStatistics.NetPerformanceStatistics;
import mscJNeuralNet.patterns.Patterns;
import mscJNeuralNet.trainer.NetTrainer;


/**
 * Beispielprogramm zum Testen der verschiedenen GUI-Komponenten
 * dieser API.
 * <p/>
 * Created on 07.06.2004
 * @author <A HREF="http://www.graviton.de">M. Serhat Cinar</A>
 * @version 07.06.2004
 */
public class TestJGuiComponents implements Observer{
	
	private JNetTrainingAlgorithmControl mNetTrainingAlgorithmControl;
	private Net mNet;
	private NetTrainer mNetTrainer;
	
	public TestJGuiComponents(){
		try{
			Patterns lPat = Patterns.load("alle-spl.txt");
			int [] layerSizes = {18, 8, 8, 3};
			mNet = new Net(layerSizes);
			INetConnector lRandNetConnector = new RandomSymmetryBreakingNetConnector();
			lRandNetConnector.connectNet(mNet, null);
			
			mNetTrainer = new NetTrainer();
			mNetTrainer.setTrainingPatterns(lPat);
			mNetTrainer.setTargetErrorType(NetPerformanceStatistics.ERRORTYPE_averageSumOfSquaredError);
			mNetTrainer.setTargetError(.01);
			mNetTrainer.setTargetCycles(1000);
			mNetTrainer.getObserverManager().addObserver(this);
			
			JFrame fMain = new JFrame();
			fMain.addWindowListener(
					new WindowAdapter() {
						public void windowClosing(WindowEvent e){
							System.exit(0);
						}
					}
			);
			
			Container content = fMain.getContentPane();
			content.setLayout(new FlowLayout());
			
			JNetInfoView lNetInfoView = new JNetInfoView();
			lNetInfoView.setNet(mNet);
			content.add(lNetInfoView);
			
			mNetTrainingAlgorithmControl = new JNetTrainingAlgorithmControl();
			mNetTrainingAlgorithmControl.getObserverManager().addObserver(this);
			content.add(mNetTrainingAlgorithmControl);
			
			JNetTrainerControl lNetTrainerControl = new JNetTrainerControl();
			lNetTrainerControl.setNetTrainer(mNetTrainer);
			content.add(lNetTrainerControl);
			
			mNetTrainingAlgorithmControl.getTrainingAlgorithm().setNet(mNet);
			mNetTrainer.setTrainingAlgorithm(mNetTrainingAlgorithmControl.getTrainingAlgorithm());
			
			JNetWeightView lNetWeightView = new JNetWeightView();
			lNetWeightView.setNet(mNet);
			content.add(lNetWeightView);
			
			JNetErrorGraphView lErrorGraph = new JNetErrorGraphView();
			lErrorGraph.setNetTrainer(mNetTrainer);
			content.add(lErrorGraph);
			
			fMain.pack();
			fMain.setVisible(true);
			
			System.out.println(Formats.format(mNet.getWeights()));
			setNet();
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}
	
	public static void main(String[] args) {
		new TestJGuiComponents();
	}
	
	private void setNet(){
		try {
			if (mNetTrainingAlgorithmControl.getTrainingAlgorithm().getNet() != mNet)
				mNetTrainingAlgorithmControl.getTrainingAlgorithm().setNet(mNet);
			if (mNetTrainer.getTrainingAlgorithm() != mNetTrainingAlgorithmControl.getTrainingAlgorithm())
				mNetTrainer.setTrainingAlgorithm(mNetTrainingAlgorithmControl.getTrainingAlgorithm());
		}
		catch (PatternDoesNotMatchNetException e) {
			e.printStackTrace();
		}
	} 
	
	/* (non-Javadoc)
	 * @see observerPattern.Observer#notify(observerPattern.Observable)
	 */
	public void notify(Observable pObservable) {
		if (pObservable == mNetTrainingAlgorithmControl){
			setNet();
		}
		else if (pObservable == mNetTrainer){
			if (mNetTrainer.hasFinished()){
				System.out.println(
					NetPerformanceReporter.getNetPerformance(
						mNetTrainer.getLastCalcualtedNetStatistics(), 
						mNetTrainingAlgorithmControl.getTrainingAlgorithm().getCycle()));
				System.out.println(Formats.format(mNet.getWeights()));
			}
		}
	}
}
