package mscJNeuralNet.examples;
import logger.Log;
import mscJNeuralNet.connectors.INetConnector;
import mscJNeuralNet.connectors.RandomNetConnector;
import mscJNeuralNet.net.Net;
import mscJNeuralNet.netPerformanceStatistics.NetPerformanceReporter;
import mscJNeuralNet.netPerformanceStatistics.NetPerformanceStatistics;
import mscJNeuralNet.netPerformanceStatistics.NetPerformanceStatisticsCalculator;
import mscJNeuralNet.patterns.Patterns;
import mscJNeuralNet.trainingAlgorithms.BackpropagationMomentum;
import mscJNeuralNet.trainingAlgorithms.INetTrainingAlgorithm;
import mscJNeuralNet.trainingAlgorithms.RProp;


/**
 * Beispiel zum Vergleich der beiden Lernverfahren 
 * {@link mscJNeuralNet.trainingAlgorithms.RProp} und 
 * {@link mscJNeuralNet.trainingAlgorithms.BackpropagationMomentum}.
 * <p/>
 * Created on 03.06.2004
 * @author <A HREF="http://www.graviton.de">M. Serhat Cinar</A>
 * @version 03.06.2004
 */
public class TestRpropVsBpg {

	public static void main(String[] args) {
		try {
			Patterns lPat = Patterns.load("alle+tonleiter-smp.txt");
			Log.getInstance().writeln(lPat.toString());
			int [] layerSizes = {15, 8, 8, 3};
			Net lNet = new Net(layerSizes);
			INetTrainingAlgorithm lAlgoRProp = new RProp();
			INetConnector lRandNetConnector = new RandomNetConnector();
			lAlgoRProp.setNet(lNet);
			double [] lConnectorParameters = {.3}; 
			lRandNetConnector.connectNet(lNet, lConnectorParameters);
			NetPerformanceStatistics lNetStat = NetPerformanceStatisticsCalculator.calculateErrors(lNet, lPat.getInputPatterns(), lPat.getOutputPatterns());
			double lSse = 1D;
			while (lAlgoRProp.getCycle() < 1000 && lSse > 0.01){
				Net.train(lPat.getInputPatterns(), lPat.getOutputPatterns(), lAlgoRProp);
				lNetStat = NetPerformanceStatisticsCalculator.calculateErrors(lNet, lPat.getInputPatterns(), lPat.getOutputPatterns());
				lSse = lNetStat.averageSumOfSquaredErrors;
			}
			Log.getInstance().writeln(NetPerformanceReporter.getNetPerformance(lNetStat, lAlgoRProp.getCycle()));
			
			double [] momentum_1_bpg = {0.001, 0.7};
			INetTrainingAlgorithm lAlgoBPG = new BackpropagationMomentum();
			lRandNetConnector.connectNet(lNet, lConnectorParameters);
			lAlgoBPG.setNet(lNet);
			lAlgoBPG.setLearningParameters(momentum_1_bpg);
			lSse = 1D;
			while (lAlgoBPG.getCycle() < 10000 && lSse > 0.01){
				Net.train(lPat.getInputPatterns(), lPat.getOutputPatterns(), lAlgoBPG);
				lNetStat = NetPerformanceStatisticsCalculator.calculateErrors(lNet, lPat.getInputPatterns(), lPat.getOutputPatterns());
				lSse = lNetStat.averageSumOfSquaredErrors;
			}
			Log.getInstance().writeln(NetPerformanceReporter.getNetPerformance(lNetStat, lAlgoBPG.getCycle()));
			
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
}
