package mscJNeuralNet.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;

import mscJNeuralNet.gui.subComponents.JGraphView;
import mscJNeuralNet.gui.subComponents.JGraphViewRuleLeft;
import mscJNeuralNet.gui.subComponents.JGraphViewRuleTop;
import mscJNeuralNet.net.Net;
import mscJNeuralNet.netPerformanceStatistics.LightWeightNetPerformanceStatistics;
import mscJNeuralNet.netPerformanceStatistics.NetPerformanceStatistics;
import mscJNeuralNet.trainer.NetTrainer;
import observerPattern.Observable;
import observerPattern.Observer;

/**
 * Grafische Komponente zur Darstellung des Fehlergraphen.
 * Der Fehlergraph stellt den Unterschied zwischen Ist-Ausgabe und Soll-Ausgabe
 * des Netzes dar.<p/>
 * <img src="../../images/jneterrorgraphview.gif"/><p/>
 * 
 * Created on 10.06.2004
 * @author <A HREF="http://www.graviton.de">M. Serhat Cinar</A>
 * @version 10.06.2004
 */
public class JNetErrorGraphView extends JPanel implements Observer{
	private static final ResourceBundle smMSCResourceBundle = ResourceBundle.getBundle("mscJNeuronalNet_Text");
	
	private Vector mLeightWeightNetStatistics;
	private NetTrainer mNetTrainer; 
	private Net mNet;
	private int mErrorType = 0;
	// eine fehlereinheit entspricht sovielen pixeln hhe
	private double mHeightScaleFactor = 200;
	private int mDefaultWidth = 400;
	
	private JComboBox mCBErrorTypes;
	private JGraphView mGraph;
	private JScrollPane mScrollPane;
	private JGraphViewRuleLeft mRuleLeft;
	private JGraphViewRuleTop mRuleTop;
	
	public JNetErrorGraphView(){
		super();
		setBorder(
			BorderFactory.createTitledBorder(
				BorderFactory.createLineBorder(Color.BLACK),
				smMSCResourceBundle.getString("ERROR_GRAPH")
			)
		);
		setPreferredSize(new Dimension(mDefaultWidth+40, (int)mHeightScaleFactor+80));
		
		mCBErrorTypes = new JComboBox(NetPerformanceStatistics.STR_errorTypes_short);
		mCBErrorTypes.addActionListener(
			new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					int lSelectedErrorType = mCBErrorTypes.getSelectedIndex();
					if (lSelectedErrorType != mErrorType){
						// change errortype
						setErrorType(lSelectedErrorType);
					}
				}
			}
		);
		add(mCBErrorTypes);
		mGraph = new JGraphView(1D, 1D/mHeightScaleFactor, 50D, 1D/8D);
		mGraph.setPreferredSize(new Dimension(mDefaultWidth, (int)Math.ceil(2*mHeightScaleFactor)));
		mScrollPane = new JScrollPane(mGraph);
		mScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		mScrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		mScrollPane.setPreferredSize(new Dimension(mDefaultWidth+20, (int)mHeightScaleFactor+20));
		mGraph.scrollRectToVisible(new Rectangle(0, (int)mHeightScaleFactor, mDefaultWidth, getPreferredSize().height));
		mRuleLeft = new JGraphViewRuleLeft(mGraph);
		mScrollPane.setRowHeaderView(mRuleLeft);
		mRuleTop = new JGraphViewRuleTop(mGraph);
		mScrollPane.setColumnHeaderView(mRuleTop);
		add(mScrollPane);
	}
	
	public void setNetTrainer(NetTrainer pNetTrainer){
		if (mNetTrainer != null) mNetTrainer.getObserverManager().removeObserver(this);
		mNetTrainer = pNetTrainer;
		mErrorType = mNetTrainer.getTargetErrorType();
		mCBErrorTypes.setSelectedIndex(mErrorType);
		mNetTrainer.getObserverManager().addObserver(this);
		reset();
	}
	
	public void reset(){
		mLeightWeightNetStatistics = new Vector();
		mGraph.reset();
		mGraph.setPreferredSize(new Dimension(mDefaultWidth, (int)Math.ceil(2*mHeightScaleFactor)));
		mScrollPane.getHorizontalScrollBar().setValue(0);
		mScrollPane.getVerticalScrollBar().setValue(mScrollPane.getVerticalScrollBar().getMaximum());
		//Let the scroll pane know to update itself
		//and its scrollbars.
		mGraph.revalidate();
		mNet = mNetTrainer.getTrainingAlgorithm().getNet();
	}
	
	public void setErrorType(int pErrorType){
		mErrorType = pErrorType;
		if (mCBErrorTypes.getSelectedIndex() != mErrorType) mCBErrorTypes.setSelectedIndex(mErrorType);
		addAllPoints();
	}
	
	private void addPoint(LightWeightNetPerformanceStatistics pLNetStat){
		mLeightWeightNetStatistics.add(pLNetStat);
		Integer lData = new Integer(Math.round((float)(pLNetStat.getErrorByType(mErrorType)*mHeightScaleFactor)));
		mGraph.addData(lData);
	}
	
	private void addAllPoints(){
		Vector lData = new Vector();
		LightWeightNetPerformanceStatistics lLNetStat;
		Integer lInt;
		for (int lX=0; lX < mLeightWeightNetStatistics.size(); lX++){
			lLNetStat = (LightWeightNetPerformanceStatistics) mLeightWeightNetStatistics.get(lX);
			lInt = new Integer(Math.round((float)(lLNetStat.getErrorByType(mErrorType)*mHeightScaleFactor)));
			lData.add(lInt);
		}
		mGraph.reset();
		mGraph.addData(lData);
	}
	
	/* (non-Javadoc)
	 * @see observerPattern.Observer#notify(observerPattern.Observable)
	 */
	public void notify(Observable pObservable) {
		if (pObservable == mNetTrainer){
			if (mNet != mNetTrainer.getTrainingAlgorithm().getNet())
				reset();
				
			NetPerformanceStatistics lNetStat = mNetTrainer.getLastCalcualtedNetStatistics();
			if (lNetStat != null){
				if (mLeightWeightNetStatistics.size()>0){
					// check, if data already is registered
					if (lNetStat != mLeightWeightNetStatistics.get(mLeightWeightNetStatistics.size()-1)){
						// new data
						addPoint(LightWeightNetPerformanceStatistics.getLightWeightNetPerformance(lNetStat));
					}
				}
				else{
					// new data
					addPoint(LightWeightNetPerformanceStatistics.getLightWeightNetPerformance(lNetStat));
				}
			}
			if (mNetTrainer.getTargetErrorType() != mErrorType) setErrorType(mNetTrainer.getTargetErrorType());
		}
	}
}
