package mscJNeuralNet.gui.subComponents;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;

import javax.swing.JComponent;

import formats.Formats;

import observerPattern.Observable;
import observerPattern.Observer;


/**
 * Ein Panel zur Darstellung des horizontalen Einheiten-Lineals fr die
 * GUI-Komponente JGraphView.
 * Diese Komponente wird benutzt, um in einem JScrollPane als Lineal
 * links die horizontalen Einheiten darzustellen.
 * <p/>
 * <img src="../../../images/jgraphviewruleleft.gif"/><p/>
 * Created on 11.06.2004
 * @author <A HREF="http://www.graviton.de">M. Serhat Cinar</A>
 * @version 11.06.2004
 * @see javax.swing.JScrollPane
 * @see mscJNeuralNet.gui.JNetErrorGraphView
 * @see mscJNeuralNet.gui.subComponents.JGraphView
 */
public class JGraphViewRuleLeft extends JComponent implements Observer{
	
	private int mWidth = 35;
	private JGraphView mGraphView;
	
	/**
	 * Erstellt ein Lineal fr die vertikalen Einheiten eines JGraphViews
	 * zur Benutzung in einem ScrollPane.
	 * 
	 * @param pGraphView Der JGraphView, zu dem die Darstellung der Einheiten erstellt werden sollen.
	 * @see javax.swing.JScrollPane
	 */
	public JGraphViewRuleLeft(JGraphView pGraphView){
		mGraphView = pGraphView;
		mGraphView.getObserverManager().addObserver(this);
		setPreferredSize(new Dimension(mWidth, mGraphView.getPreferredSize().height));
	}
	
	/* (non-Javadoc)
	 * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
	 */
	public void paintComponent(Graphics pG){
		int lYPos;
		double lScaleNum = 0D;
		// horizontal grid
		lScaleNum = 0;
		pG.setColor(Color.BLACK);
		double lHeightGridsPixelStep = mGraphView.getHeightPixelStep();
		double lHeightGridsEvery = mGraphView.getHeightGridsEvery();
		for (double lY = 0; lY < getPreferredSize().height; lY += lHeightGridsPixelStep){
			lYPos = getPreferredSize().height-Math.round((float)lY);
			pG.drawString(Formats.format(lScaleNum, 3), 0, lYPos);
			lScaleNum += lHeightGridsEvery;
		}
	}
	
	/* (non-Javadoc)
	 * @see observerPattern.Observer#notify(observerPattern.Observable)
	 */
	public void notify(Observable pObservable) {
		if (pObservable == mGraphView){
			if (getPreferredSize().height != mGraphView.getPreferredSize().height)
				setPreferredSize(new Dimension(mWidth, mGraphView.getPreferredSize().height));
		}
	}
}
