package mscJNeuralNet.gui.subComponents;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;

import javax.swing.JComponent;

import formats.Formats;

import observerPattern.Observable;
import observerPattern.Observer;

/**
 * Ein Panel zur Darstellung des vertikalen Einheiten-Lineals fr die
 * GUI-Komponente JGraphView.
 * Diese Komponente wird benutzt, um in einem JScrollPane als Lineal
 * oben die vertikale Einheiten darzustellen.
 * <p/>
 * <img src="../../../images/jgraphviewruletop.gif"/><p/>
 * Created on 11.06.2004
 * @author <A HREF="http://www.graviton.de">M. Serhat Cinar</A>
 * @version 11.06.2004
 * @see javax.swing.JScrollPane
 * @see mscJNeuralNet.gui.JNetErrorGraphView
 * @see mscJNeuralNet.gui.subComponents.JGraphView
 */
public class JGraphViewRuleTop extends JComponent implements Observer{
	
	private JGraphView mGraphView;
	private int mHeight = 25;
	
	/**
	 * Erstellt ein Lineal fr die vertikalen Einheiten eines JGraphViews
	 * zur Benutzung in einem ScrollPane.
	 * 
	 * @param pGraphView Der JGraphView, zu dem die Darstellung der Einheiten erstellt werden sollen.
	 * @see javax.swing.JScrollPane
	 */
	public JGraphViewRuleTop(JGraphView pGraphView){
		mGraphView = pGraphView;
		mGraphView.getObserverManager().addObserver(this);
		setPreferredSize(new Dimension(mGraphView.getPreferredSize().width, mHeight));
	}
	
	/* (non-Javadoc)
	 * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
	 */
	public void paintComponent(Graphics pG){
		int lXPos;
		double lScaleNum = 0D;
		// vertical grids
		pG.setColor(Color.BLACK);
		double lWidthGridsPixelStep = mGraphView.getWidthPixelStep();
		double lWidthGridsEvery = mGraphView.getWidthGridsEvery();
		for (double lX = 0; lX < getPreferredSize().width; lX += lWidthGridsPixelStep){
			lXPos = Math.round((float)lX);
			pG.drawString(Formats.format(lScaleNum, 2), lXPos, getPreferredSize().height);
			lScaleNum += lWidthGridsEvery;
		}
	}
	
	/* (non-Javadoc)
	 * @see observerPattern.Observer#notify(observerPattern.Observable)
	 */
	public void notify(Observable pObservable) {
		if (pObservable == mGraphView){
			if (getPreferredSize().width != mGraphView.getPreferredSize().width)
				setPreferredSize(new Dimension(mGraphView.getPreferredSize().width, mHeight));
		}
	}
}
