package mscJNeuralNet.gui.subComponents;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import observerPattern.Observable;
import observerPattern.ObserverManager;

/**
 * Eine JComboBox mit einem JLabel.
 * ber die Schnittstelle {@link observerPattern.Observable}
 * kann dieses Control berwacht werden.
 * Bei der nderung der Auswahlbox werden alle Observer benachrichtigt. 
 * <p/>
 * Created on 16.06.2004
 * @author <A HREF="http://www.graviton.de">M. Serhat Cinar</A>
 * @version 16.06.2004
 * @see javax.swing.JComboBox
 * @see javax.swing.JLabel
 * @see observerPattern.Observable
 */
public class JLabeledComboBox extends JPanel implements Observable{
	private JLabel mLabel;
	private JComboBox mComboBox;
	private ObserverManager mObserverManager = new ObserverManager();
	private JLabeledComboBox mThis;

	/**
	 * Erzeugt eine JComboBox mit einem JLabel. Die bergebenen Objekte
	 * werden der ComboBox hinzugefgt.
	 * 
	 * @param pComboBoxObjects Objekte, die der JComboBox hinzugefgt werden sollen.
	 */
	public JLabeledComboBox(Object [] pComboBoxObjects){
		super();
		mLabel = new JLabel();
		add(mLabel);
		mThis = this;
		
		mComboBox = new JComboBox(pComboBoxObjects);
		mComboBox.addActionListener(
			new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					getObserverManager().notifyObservers(mThis);
				}
			}
		);
		add(mComboBox);
	}
	
	/**
	 * Liefert das JLabel dieser Komponente zurck.
	 * 
	 * @return Das JLabel dieser Komponente.
	 */
	public JLabel getLabel(){
		return mLabel;
	}
	
	/**
	 * Liefert die JComboBox dieser Komponente zurck.
	 * 
	 * @return Die JComboBox dieser Komponente.
	 */
	public JComboBox getComboBox(){
		return mComboBox;
	}
	
	/* (non-Javadoc)
	 * @see observerPattern.Observable#getObserverManager()
	 */
	public ObserverManager getObserverManager() {
		return mObserverManager;
	}

}
