package mscJNeuralNet.gui.subComponents;

import java.awt.Color;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import observerPattern.Observable;
import observerPattern.ObserverManager;

/**
 * Ein JTextField mit einem JLabel.
 * ber die Schnittstelle {@link observerPattern.Observable}
 * kann dieses Control berwacht werden.
 * Beim verlassen des JTextField oder beim Drcken der Returntaste
 * im Textfeld werden alle Observer benachrichtigt. 
 * <p/>
 * Created on 04.06.2004
 * @author <A HREF="http://www.graviton.de">M. Serhat Cinar</A>
 * @version 22.06.2004
 * @see javax.swing.JTextField
 * @see javax.swing.JLabel
 * @see observerPattern.Observable
 */
public class JLabeledTextField extends JPanel implements Observable{
	private JLabel mLabel;
	private JTextField mTextField;
	private ObserverManager mObserverManager = new ObserverManager();
	private static final char CHAR_RETURN = '\n';
	private JLabeledTextField mThis;
	private boolean mIsNotifying = false; 

	/**
	 * Erzeugt ein JTextField mit einem JLabel.
	 * Die Gre des JTextField wird auf 0 gesetzt.
	 */
	public JLabeledTextField(){
		this(0);
	}
	
	/**
 	 * Erzeugt ein JTextField mit einem JLabel.
	 * Die Gre (Anzahl Zeichen) des JTextField wird auf den gegebenen Parameter gesetzt.
	 * 
	 * @param pTextFieldSize Die Gre des JTextFields in Zeichen.
	 */
	public JLabeledTextField(int pTextFieldSize){
		super();
		mLabel = new JLabel();
		add(mLabel);
		mThis = this;
		
		if (pTextFieldSize >= 0)
			mTextField = new JTextField(pTextFieldSize);
		else mTextField = new JTextField();
		
		mTextField.setBackground(Color.white);
		mTextField.addKeyListener(
			new KeyAdapter(){
				public void keyTyped(KeyEvent evt){
					if (evt.getKeyChar() == CHAR_RETURN)
						if (!mIsNotifying){
							mIsNotifying = true;
							getObserverManager().notifyObservers(mThis);
							mIsNotifying = false;
						}
				}
			}
		);
		mTextField.addFocusListener(
			new FocusAdapter(){
				public void focusLost(FocusEvent e){
					if (!mIsNotifying){
						mIsNotifying = true;
						getObserverManager().notifyObservers(mThis);
						mIsNotifying = false;
					}
				}
			}
		);
		add(mTextField);
	}
	
	/**
	 * Liefert das JLabel dieser Komponente zurck.
	 * 
	 * @return Das JLabel dieser Komponente.
	 */
	public JLabel getLabel(){
		return mLabel;
	}
	
	/**
	 * Liefert das JTextField dieser Komponente zurck.
	 * 
	 * @return Das JTextField dieser Komponente.
	 */
	public JTextField getTextField(){
		return mTextField;
	}
	
	/* (non-Javadoc)
	 * @see observerPattern.Observable#getObserverManager()
	 */
	public ObserverManager getObserverManager() {
		return mObserverManager;
	}
}
