package mscJNeuralNet.netPerformanceStatistics;

/**
 * Diese Klasse enthlt Informationen zu verschiedenen Netzfehlern.
 * Die verfgbaren Netzfehlertypen werden in der 
 * Klasse {@link mscJNeuralNet.netPerformanceStatistics.NetPerformanceStatistics} festgelegt.
 * Diese Klasse soll zum speicherfreundlichen Speichern der
 * bentigten durchschnitts Fehlerwerte dienen. 
 * <p/>
 * Eine Instanz dieser Klasse wird aus einer Instanz von
 * {@link mscJNeuralNet.netPerformanceStatistics.NetPerformanceStatistics} erzeugt:<br/>
 * <pre>
 * // Netz erzeugen und initialisieren
 * int [] lLayerSizes = {2, 1};
 * Net myNet = new Net(lLayerSizes);
 * new RandomSymmetryBreakingNetConnector().connectNet(myNet, null);
 * 
 * // Lerndaten fr Xor 
 * double [][] lXorInput = {{1,1}, {1,-1}, {-1, 1}, {-1,-1}};
 * double [][] lXorOutput = {{-1}, {1}, {1}, {-1}};
 * 
 * // Aktuelle Netzperformaz berechnen
 * NetPerformanceStatistics lActualStats = 
 * 	NetPerformanceStatisticsCalculator.calculateErrors(
 * 		myNet, lXorInput, lXorOutput);
 * 
 * LightWeightNetPerformanceStatistics lLightStats =
 * 	LightWeightNetPerformanceStatistics.getLeightWeightNetPerformance(lActualStats);
 * 
 * System.out.println(STR_averageSumAbsoluteError_short+"="+
 * 	Double.toString(lLightStats.averageSumOfAbsoluteErrors));
 * </pre>
 * <p/>
 * Created on 19.06.2004
 * @author <A HREF="http://www.graviton.de">M. Serhat Cinar</A>
 * @version 19.06.2004
 * @see mscJNeuralNet.netPerformanceStatistics.NetPerformanceStatisticsCalculator
 * @see mscJNeuralNet.netPerformanceStatistics.NetPerformanceStatistics
 * @see mscJNeuralNet.netPerformanceStatistics
 */
public class LightWeightNetPerformanceStatistics {

	/**
	 * Durchschnitt der Summe aller absoluten Fehlerbetrge (avg. ABSE)
	 */
	public double averageSumOfAbsoluteErrors;

	/**
	 * Durchschnitt der Fehlerquadratsummen aller Lerndatenstze (avg. SSE)
	 */
	public double averageSumOfSquaredErrors;
		
	/**
	 * Durchschnitt der Quadratwurzeln der quadratischen
	 * Fehler aller Lerndatenstze (avg. RMSE) 
	 */
	public double averageRootMeanSquareError;
	
	/**
	 * Der grte absolute Fehlerwert eines einzelnen Neurons,
	 * berechnet als Abs(Soll-Wert - Ist-Wert).
	 */
	public double maximumNeuronAbsoluteError;
	
	/**
	 * Liefert den Fehlerwert, der durch den bergebenen Parameter (ERRORTYPE)
	 * festgelegt wurde.
	 * 
	 * @param pErrorType Eine Konstante ERRORTYPE fr den Fehlertypen asu der Klasse NetPerformanceStatistics.
	 * @return Wert des Fehlers.
	 */
	public double getErrorByType(int pErrorType){
		switch (pErrorType){
			case NetPerformanceStatistics.ERRORTYPE_averageSumAbsoluteError:
				return averageSumOfAbsoluteErrors;
			case NetPerformanceStatistics.ERRORTYPE_averageSumOfSquaredError:
				return averageSumOfSquaredErrors;
			case NetPerformanceStatistics.ERRORTYPE_averageRootMeanSquareError:
				return averageRootMeanSquareError;
			case NetPerformanceStatistics.ERRORTYPE_maximumNeuronAbsoluteError:
				return maximumNeuronAbsoluteError;
		}
		return 0;
	}
	
	/**
	 * Erzeugt ein LightWeightNetPerformanceStatistics aus einem NetPerformanceStatistics
	 * Objekt.
	 * 
	 * @param pNetStat Das zugrundeliegende NetPerformanceStatistics Objekt, aus dem die
	 * 			 LightWeightNetPerformanceStatistics erzeugt werden soll.
	 * @return LightWeightNetPerformanceStatistics Objekt mit den aktuellen Werten.
	 */
	public static LightWeightNetPerformanceStatistics getLightWeightNetPerformance(NetPerformanceStatistics pNetStat){
		LightWeightNetPerformanceStatistics lLNetStat = new LightWeightNetPerformanceStatistics();
		lLNetStat.averageRootMeanSquareError = pNetStat.averageRootMeanSquareError;
		lLNetStat.averageSumOfAbsoluteErrors = pNetStat.averageSumOfAbsoluteErrors;
		lLNetStat.averageSumOfSquaredErrors = pNetStat.averageSumOfSquaredErrors;
		lLNetStat.maximumNeuronAbsoluteError = pNetStat.maximumNeuronAbsoluteError;
		return lLNetStat;
	}
}
