package observerPattern;

import java.util.Vector;

/**
 * Der ObserverManager verwaltet alle Observer fr einen Observable.
 * Jede Klasse, die Observable implementiert, muss einen eigenen
 * ObserverManager haben.
 * Alle Observer, die von einem Observable benachrichtigt werden sollen,
 * mssen sich beim ObserverManager des Observables anmelden:<br/>
 * <code>theObservable.getObserverManager().addObserver(this);</code>
 * <p/>
 * Created on 21.04.2004
 * @author <A HREF="http://www.graviton.de">M. Serhat Cinar</A>
 * @version 21.04.2004
 * @see observerPattern.Observable
 * @see observerPattern.Observer
 */
public class ObserverManager {
	private Vector mObservers;
	
	/**
	 * ber diese Methode kann sich ein Observer bei einem Observable registrieren,
	 * um von diesem benachrichtigt zu werden.
	 * 
	 * @param pObserver Der anzumeldende Observer.
	 * @see observerPattern.Observable
	 * @see observerPattern.Observer
	 */
	public void addObserver(Observer pObserver){
		if (mObservers == null) mObservers = new Vector();
		if (!mObservers.contains(pObserver)) mObservers.add(pObserver);
	}
	
	/**
	 * ber diese Methode kann sich ein registrierter Observer bei einem Observable wieder
	 * abmelden, um von diesem nicht mehr benachrichtigt zu werden.
	 * 
	 * @param pObserver Der abzumeldende Observer.
	 * @see observerPattern.Observable
	 * @see observerPattern.Observer
	 */
	public void removeObserver(Observer pObserver){
		if (mObservers != null)
			if (mObservers.contains(pObserver)) mObservers.remove(pObserver);
	}
	
	/**
	 * Benachrichtigt alle registrierten Observer, die von dem Observable
	 * benachrichtigt werden soll. 
	 * 
	 * @param pObservable Der Observer, der die registrierten Observer benachrichtigen mchte.
	 * @see observerPattern.Observable
	 * @see observerPattern.Observer
	 */
	public void notifyObservers(Observable pObservable){
		if (mObservers != null)
			for (int i=0; i<mObservers.size(); i++)
				((Observer) mObservers.get(i)).notify(pObservable);
	}
}
